/*
 * Copyright (C) 2014 Yahoo Inc.
 */

// The number of children present when there are no lozenges or data in the address field
var EMPTY_ADDRESS_FIELD_COUNT = 2;

/**
 * An object which controls the show/hide behavior of empty CC/BCC and From fields
 */
var ccBccFromSummaryFieldController = {
    toRowNode: null,
    toLabelNode: null,
    toLozengeContainerNode: null,
    toRowChevronNode: null,
    toRowChevronIconNode: null,
    recipientSummaryNode: null,
    headerNode: null,
    subjectInputNode: null,
    contentNode: null,
    ccRowNode: null,
    ccInputNode: null,
    bccInputNode: null,
    bccRowNode: null,
    fromNode: null,
    replyToNode: null,
    fromAddressTextNode: null,
    replyToTextNode: null,
    replyToImageNode: null,
    ccLozengeContainerNode: null,
    bccLozengeContainerNode: null,
    toInputNodeId: null,
    ccInputNodeId: null,
    bccInputNodeId: null,
    toInputUniqueTag: null,
    ccInputUniqueTag: null,
    bccInputUniqueTag: null,
    isCurrentlyExpanded: null,
    isDarkMode: null,
    allRecipients: null,

    /**
     * A method which starts the ccBccFromSummaryFieldController
     */
    init: function(toRowNode, toLabelNode, toLozengeContainerNode, toRowChevronNode, toRowChevronIconNode, recipientSummaryNode, headerNode, subjectInputNode, contentNode, ccRowNode, ccInputNode, bccInputNode, bccRowNode, fromNode, replyToNode, fromAddressTextNode, replyToTextNode, replyToImageNode, ccLozengeContainerNode, bccLozengeContainerNode, toInputNodeId, ccInputNodeId, bccInputNodeId, toInputUniqueTag, ccInputUniqueTag, bccInputUniqueTag) {
        this.toRowNode = toRowNode;
        this.toLabelNode = toLabelNode;
        this.toLozengeContainerNode = toLozengeContainerNode;
        this.toRowChevronNode = toRowChevronNode;
        this.toRowChevronIconNode = toRowChevronIconNode;
        this.recipientSummaryNode = recipientSummaryNode;
        this.headerNode = headerNode;
        this.subjectInputNode = subjectInputNode;
        this.contentNode = contentNode;
        this.ccRowNode = ccRowNode;
        this.ccInputNode = ccInputNode;
        this.bccInputNode = bccInputNode;
        this.bccRowNode = bccRowNode;
        this.fromNode = fromNode;
        this.replyToNode = replyToNode;
        this.fromAddressTextNode = fromAddressTextNode;
        this.replyToTextNode = replyToTextNode;
        this.replyToImageNode = replyToImageNode;
        this.ccLozengeContainerNode = ccLozengeContainerNode;
        this.bccLozengeContainerNode = bccLozengeContainerNode;
        this.toInputNodeId = toInputNodeId;
        this.ccInputNodeId = ccInputNodeId;
        this.bccInputNodeId = bccInputNodeId;
        this.toInputUniqueTag = toInputUniqueTag;
        this.ccInputUniqueTag = ccInputUniqueTag;
        this.bccInputUniqueTag = bccInputUniqueTag;
        this.isDarkMode = yMailBridge.isDarkThemeEnabled();
        this.isCurrentlyExpanded = false;

        // FIXME potential for memory leaks if init is called more than once.
        this.toRowChevronNode.addEventListener('click', this.toggleSummary.bind(this), true);
        this.headerNode.addEventListener('focusin', this.headerFocusHandler.bind(this));
        this.toRowNode.addEventListener('click', this.handleSummaryClick.bind(this), true);
        this.contentNode.addEventListener('focusin', this.headerFocusHandler.bind(this));
    },

    handleSummaryClick: function(event) {
        // Make sure that interactions on lozenges themselves don't result the To field expanding
        if (event.target.classList.contains("lozenge-element")) {
            return;
        }

        if (!this.isCurrentlyExpanded) {
            this.expandSummaryIntoDetailView();
            event.stopPropagation();
        }
    },
    /**
     * Sets the From address text. Also recomputes the summary text by fetching it from the native code
     */
    setFromAddress: function(address) {
        this.fromAddressTextNode.textContent = address;
    },

    /**
     * Sets the Reply to address text and icon according to theme and address types(verified/unverified/primary)
     */
    showReplyToField: function(show, address, isVerified, isPrimary, warningIconLabel) {
        if (show) {
            var defaultReplyToNode = document.getElementById(DEFAULT_REPLY_TO_ADDRESS_ELEMENT_ID);
            this.replyToTextNode.textContent = decodeURIComponent(address);
            defaultReplyToNode.textContent = decodeURIComponent(address);
            this.replyToNode.classList.remove("hidden");
            this.replyToImageNode.setAttribute('aria-label', warningIconLabel);
            var textStyle = "default-text-style";
            if (isVerified) {
                if (this.isDarkMode) {
                    textStyle = "reply-to-text-style-important-dark";
                    this.replyToImageNode.setAttribute("src", "./images/important_icon_dark.svg");
                } else {
                    textStyle = "reply-to-text-style-important-light";
                    this.replyToImageNode.setAttribute("src", "./images/important_icon_light.svg");
                }
            } else {
                if (this.isDarkMode) {
                    textStyle = "reply-to-text-style-caution-light";
                    this.replyToImageNode.setAttribute("src", "./images/caution_icon.svg");
                } else {
                    textStyle = "reply-to-text-style-caution-dark";
                    this.replyToImageNode.setAttribute("src", "./images/caution_icon_light.svg");
                }
            }

            if (isPrimary) {
                this.replyToImageNode.classList.add("hidden");
                this.replyToTextNode.classList.add("hidden");
                defaultReplyToNode.classList.remove("hidden");
            } else {
                defaultReplyToNode.classList.add("hidden");
                this.replyToImageNode.classList.remove("hidden");
                this.replyToTextNode.classList.remove("hidden");
                this.replyToTextNode.classList.remove("default-text-style");
                this.replyToTextNode.classList.add(textStyle);
            }
        } else {
            this.replyToNode.classList.add("hidden");
        }
    },

    /**
     * Depending on the state of the row, expand or collapse the CC/BCC rows accordingly
     */
    toggleSummary: function(fromChevron) {
        if (this.isCurrentlyExpanded) {
            this.collapseDetailIntoSummaryView(fromChevron)
        } else {
            this.expandSummaryIntoDetailView()
        }
    },

    /**
     * Expands the summary node into individual CC/BCC/From fields
     * @param setFocusFieldId an optional string parameter specifying the ID of the CC/BCC field onto which focus will be set after expanding the summary
     */
    expandSummaryIntoDetailView: function() {
        this.ccRowNode.classList.remove("hidden");
        this.bccRowNode.classList.remove("hidden");
        this.fromNode.classList.remove("hidden");
        this.toRowChevronIconNode.setAttribute("src", "./images/chevron_up.svg");
        this.toRowChevronNode.setAttribute("aria-expanded", true);
        yMailBridge.setCCBCCFromRecipientSummaryFieldExpandState(true);
        this.isCurrentlyExpanded = true;
        // Hide the summary element and then show all lozenges again when expanded
        this.toLabelNode.classList.remove("hidden");
        this.recipientSummaryNode.classList.add("hidden");
        lozengeController.showAllLozenges(this.toInputUniqueTag);
    },

    /**
     * Collapses the detail view into a summary view
     */
    collapseDetailIntoSummaryView: function(fromChevron) {
        // Only show if there are > 2 recipients in total
        var toRecipients = lozengeController.getRecipientNamesForField(this.toInputUniqueTag);
        var ccRecipients = lozengeController.getRecipientNamesForField(this.ccInputUniqueTag);
        var bccRecipients = lozengeController.getRecipientNamesForField(this.bccInputUniqueTag);
        var allRecipientsArray = toRecipients.concat(ccRecipients).concat(bccRecipients);
        var totalRecipientCount = toRecipients.length + ccRecipients.length + bccRecipients.length;
        // Check if conditions for showing summary line are fulfilled
        if (totalRecipientCount > 0) {
            // Create a canvas to measure out width of text
            var canvas = document.createElement("canvas");
            var ctx = canvas.getContext("2d");
            ctx.font = "13px sans-serif";
            var availableRowWidth = this.toRowNode.offsetWidth - 80;
            var finalizedSummaryText;

            // Greedily try to show the maximum number of recipients possible
            // First try to see if it's possible to hide none of the names
            // Otherwise iteratively increase the number of names hidden, stopping as soon as it fits
            // on one line
            for (var i = totalRecipientCount; i >= 1; i--) {
                // Create a list of recipients with a separator between (comma in English)
                var summaryTextCandidate = "";
                var shownRecipients = allRecipientsArray.slice(0, i);
                var hiddenRecipientCount = totalRecipientCount - i;
                summaryTextCandidate = yMailBridge.getRecipientSummary(JSON.stringify(shownRecipients), hiddenRecipientCount);
                // Check if the text will fit on the row
                if (ctx.measureText(summaryTextCandidate).width < availableRowWidth) {
                    finalizedSummaryText = summaryTextCandidate;
                    break;
                }
            }
            // If summary text is undefined, then even one recipient is too much to show, in that case
            // display the N recipients variant of the summary
            if (!finalizedSummaryText) {
                finalizedSummaryText = decodeURIComponent(yMailBridge.getNRecipientsSummaryLine(totalRecipientCount));
            }

            // Update and show summary line first before showing
            // Hide all lozenges from lozenge manager
            this.recipientSummaryNode.textContent = finalizedSummaryText;
            lozengeController.hideAllLozenges(this.toInputUniqueTag);
            this.toLabelNode.classList.add("hidden");
            this.recipientSummaryNode.classList.remove("hidden");
            // Related contacts shouldn't be shown when collapsed
            contactRelatedController.close();
        }

        this.toRowChevronIconNode.setAttribute("src", "./images/chevron_down.svg");
        this.toRowChevronNode.setAttribute("aria-expanded", false);
        this.ccRowNode.classList.add("hidden");
        this.bccRowNode.classList.add("hidden");
        this.fromNode.classList.add("hidden");
        this.isCurrentlyExpanded = false;

        // If the src of this collapse is from the chevron, then we should handle the "auto-focus" onto
        // the next relevant element - otherwise, don't overwrite user intentions
        if (fromChevron) {
            // Check if Subject has been filled - if it has been then focus on Compose content node otherwise, focus on Subject
            if (this.subjectInputNode.value.trim().length === 0) {
                this.subjectInputNode.focus();
            } else {
                this.contentNode.focus();
            }
        }

        yMailBridge.setCCBCCFromRecipientSummaryFieldExpandState(false);
    },

    /**
     * Computes the state of the summary field everytime the focus switched between various fields in the compose view
     */
    headerFocusHandler: function(event) {
        // (1) Don't collapse for any of the fields To, Cc or Bcc (those being expanded)
        // (2) Don't collapse when the user interacts with the lozenge elements (remove button or tapping on lozenge itself)
        // (3) For keyboard users, if they focus on chevron node - don't collapse To field
        if (event.target.id !== this.toInputNodeId &&
            event.target.id !== this.ccInputNodeId &&
            event.target.id !== this.bccInputNodeId &&
            !event.target.classList.contains("lozenge-element") &&
            event.target.id !== this.toRowChevronNode.id) {
            // For some version of webview, the focus event will happen first and then the click event will go through.
            // In this case, if we collapse the cc/bcc here, the click event will be triggered at the message body container.
            // Then it will focus the message body afterwards.
            // This is a work around. We collapse the cc/bcc after current event handling cycle.
            setTimeout(this.collapseDetailIntoSummaryView.bind(this), 0);
            // Close all search controllers when we focus on non-recipient related elements
            contactSearchController.close();
        }
    },
};